/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Draw 3D bars according to pixel colors of a given image. Manipulate view
 * by dragging mouse with left mouse button pressed and mouse wheel. Manipulate
 * bar height and width by dragging mouse with right button pressed. Bar height
 * depends on pixel brightness.
 *****************************************************************************/

PImage sourceImg;  // Processing image object

int mode = 0;      // mode index
int numModes = 4;

float stepsize = 6.0;         // distance between bars
float barsize = 20.0;         // width of bars
float maxbarheight = 1000.0;  // maximum bar height

int downsampling = 8;         // image downsampling: only use every n-th pixel (n = downsampling)

float offsetX;                // offset variables to center bar grid around origin
float offsetY;                 

float rotX = 0;               // current rotation around x-axis
float rotY = 0;               // current rotation around y-axis
float scaling = 0.6;          // current scaling

void setup() {
  size(800, 600, P3D);
  
  sourceImg = loadImage("lena.png"); // load image and create PImage object.
                                     // "lena.png" must be in the sketch's "data" directory
                                     
  offsetX = sourceImg.width*0.5*stepsize;  
  offsetY = sourceImg.height*0.5*stepsize;
}


void draw() {
  color clr;
  float barheight;
  
  
  // compute changes in rotation or barsize using changes in mouse position
  if (mousePressed == true) {
    if (mouseButton == LEFT) {
      rotX -= (mouseY - pmouseY)/(float)height*PI;
      rotY += (mouseX - pmouseX)/(float)width*PI;
    }
    else if (mouseButton == RIGHT) {
      
      maxbarheight += (mouseY - pmouseY)/(float)height*3000;
      if (maxbarheight > 3000.0)
        maxbarheight = 3000.0;
      else if (maxbarheight < 10.0)
        maxbarheight = 10.0;
   
      barsize += (mouseX - pmouseX)/(float)width*40.0;
      if (barsize > 40.0)
        barsize = 40.0;
      else if (barsize < 4.0)
        barsize = 4.0;
    }
  }
  
  background(0);
  
  translate(width/2, height/2, -1500); 
  rotateX(rotX);   
  rotateY(rotY);
  scale(scaling);
  
  for (int j = 0; j < sourceImg.height; j += downsampling) {
    for (int i = 0; i < sourceImg.width; i += downsampling) {
      clr = sourceImg.get(i, j);  // get pixel color
      barheight = (red(clr) + green(clr) + blue(clr))/(3*255)*maxbarheight;  // compute barheight from pixel color
      
      noStroke();
      fill(clr);
      
      pushMatrix();
      translate(i*stepsize - offsetX, j*stepsize - offsetY, barheight*0.5);
      box(barsize, barsize, barheight);
      popMatrix();
    }
  }
}

void mouseWheel(MouseEvent event) {
  // update scaling from mouse wheel
  scaling +=  0.05*event.getCount();
  if (scaling > 2.0)
    scaling = 2.0;
  else if (scaling < 0.5)
    scaling = 0.5;
}
